/*
 * imagesource.h - base class for the efficient scanline-based
 * handling of extremely large images.
 *
 * Copyright (c) 2004 by Alastair M. Robinson
 * Distributed under the terms of the GNU General Public License -
 * see the file named "COPYING" for more details.
 *
 */

#ifndef IMAGESOURCE_H
#define IMAGESOURCE_H

#include <stdlib.h>

typedef unsigned short ISDataType;
#define IS_SAMPLEMAX 65535
#define EIGHTTOIS(x) (((x) << 8) | (x))
#define ISTOEIGHT(x) ((((x) * 65281 + 8388608) >> 24) & 0xff)

/* Note: 0 is Black for RGB images, but White for Greyscale and CMYK images */

enum IS_TYPE {
	IS_TYPE_NULL=0,
	IS_TYPE_BW,
	IS_TYPE_GREY,
	IS_TYPE_RGB,
	IS_TYPE_CMYK,
	IS_TYPE_NULLA=8,
	IS_TYPE_BWA,
	IS_TYPE_GREYA,
	IS_TYPE_RGBA,
	IS_TYPE_CMYKA
};

#define IS_TYPE_ALPHA 8
#define STRIP_ALPHA(x) ((x)&~IS_TYPE_ALPHA)
#define HAS_ALPHA(x) ((x)&IS_TYPE_ALPHA)

class CMSProfile;
class ImageSource
{
	public:
	ImageSource() : rowbuffer(NULL)
	{
	}
	ImageSource(ImageSource *src) : rowbuffer(NULL)
	{
		width=src->width;
		height=src->height;
		type=src->type;
		samplesperpixel=src->samplesperpixel;
		xres=src->xres;
		yres=src->yres;
		randomaccess=src->randomaccess;
		embeddedprofile=src->embeddedprofile;
		currentrow=-1;
	}
	virtual ~ImageSource()
	{
		if(rowbuffer)
			free(rowbuffer);
	}
	virtual ISDataType *GetRow(int row)=0;
	void MakeRowBuffer()
	{
		rowbuffer=(ISDataType *)malloc(sizeof(ISDataType)*width*samplesperpixel);
		currentrow=-1;
	}
	void SetResolution(double xr,double yr)
	{
		xres=xr;
		yres=yr;
	}
	int width,height;
	enum IS_TYPE type;
	int samplesperpixel;
	double xres,yres;
	bool randomaccess;
	CMSProfile *embeddedprofile;
	protected:
	int currentrow;
	ISDataType *rowbuffer;
};


#endif
