/*
 * pixbuf_from_imagesource.cpp
 * Creates a GdkPixbuf from an ImageSource
 *
 * Copyright (c) 2005 by Alastair M. Robinson
 * Distributed under the terms of the GNU General Public License -
 * see the file named "COPYING" for more details.
 *
 */

#include <iostream>

#include "pixbuf_from_imagesource.h"

using namespace std;

GdkPixbuf *pixbuf_from_imagesource(ImageSource *is)
{
	if(!is)
		return(NULL);

	GdkPixbuf *pb;

	switch(IS_TYPE_RGB)
	{
		case IS_TYPE_RGB:
		case IS_TYPE_RGBA:
		case IS_TYPE_GREY:
		case IS_TYPE_GREYA:
			pb=gdk_pixbuf_new(GDK_COLORSPACE_RGB,FALSE,8,is->width,is->height);
			break;
		default:
			return(NULL);
			break;
	}

	if(pb)
	{
		int rowstride=gdk_pixbuf_get_rowstride(pb);
		unsigned char *pixels=gdk_pixbuf_get_pixels(pb);

		switch(is->type)
		{
			case IS_TYPE_RGBA:
				for(int y=0;y<is->height;++y)
				{
					ISDataType *src=is->GetRow(y);
					for(int x=0;x<is->width;++x)
					{
						int a=255-ISTOEIGHT(src[x*4+3]);
						pixels[x*3]=(ISTOEIGHT(src[x*4])*a+255*(255-a))/255;
						pixels[x*3+1]=(ISTOEIGHT(src[x*4+1])*a+255*(255-a))/255;
						pixels[x*3+2]=(ISTOEIGHT(src[x*4+2])*a+255*(255-a))/255;
					}		
					pixels+=rowstride;	
				}
				break;
			case IS_TYPE_GREYA:
				for(int y=0;y<is->height;++y)
				{
					ISDataType *src=is->GetRow(y);
					for(int x=0;x<is->width;++x)
					{
						int a=255-ISTOEIGHT(src[x*2+3]);
						pixels[x*3]=(ISTOEIGHT(src[x*2])*a+255*(255-a))/255;
						pixels[x*3+1]=(ISTOEIGHT(src[x*2])*a+255*(255-a))/255;
						pixels[x*3+2]=(ISTOEIGHT(src[x*2])*a+255*(255-a))/255;
					}		
					pixels+=rowstride;	
				}
				break;
			case IS_TYPE_RGB:
				for(int y=0;y<is->height;++y)
				{
					ISDataType *src=is->GetRow(y);
					for(int x=0;x<is->width*is->samplesperpixel;++x)
					{
						pixels[x]=ISTOEIGHT(src[x]);
					}		
					pixels+=rowstride;	
				}
				break;
			case IS_TYPE_GREY:
				for(int y=0;y<is->height;++y)
				{
					ISDataType *src=is->GetRow(y);
					for(int x=0;x<is->width*is->samplesperpixel;++x)
					{
						pixels[x*3]=ISTOEIGHT(src[x]);
						pixels[x*3+1]=ISTOEIGHT(src[x]);
						pixels[x*3+2]=ISTOEIGHT(src[x]);
					}		
					pixels+=rowstride;	
				}
				break;
			default:
				return(NULL);
				break;
		}
	}
	return(pb);
}
