#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>

#include "config.h"

#ifdef HAVE_LIBCUPS
#include <cups/cups.h>
#endif

#include "printerqueues.h"


class PrinterInstance
{
	public:
	PrinterInstance(PrinterQueues &list,const char *name) : next(NULL), list(list)
	{
		if((prev=list.first))
		{
			while(prev->next)
				prev=prev->next;
			prev->next=this;
		}
		else
			list.first=this;
		this->name=strdup(name);
		++list.printercount;
	}
	~PrinterInstance()
	{
		if(next)
			next->prev=prev;
		if(prev)
			prev->next=next;
		else
			list.first=next;
		if(name)
			free(name);
	}
	char *name;
	protected:
	PrinterInstance *next;
	PrinterInstance *prev;
	PrinterQueues &list;
	friend class PrinterQueues;
};


PrintSystem PrinterQueues::PrintSystems[] =
{
  { "lp -s", "-d", "-oraw", "/usr/sbin/cupsd",
    "/usr/bin/lpstat -v | grep -i '^device for ' | awk '{print $3}' | sed 's/://'" },
  { "lp -s", "-d", "-oraw", "/usr/bin/lp",
    "/usr/bin/lpstat -v | grep -i '^device for ' | awk '{print $3}' | sed 's/://'" },
  { "lpr", "-P", "-l", "/etc/lpc",
    "/etc/lpc status | grep '^...*:' | sed 's/:.*//'" },
  { "lpr", "-P", "-l", "/usr/bsd/lpc",
    "/usr/bsd/lpc status | grep '^...*:' | sed 's/:.*//'" },
  { "lpr", "-P", "-l", "/usr/etc/lpc",
    "/usr/etc/lpc status | grep '^...*:' | sed 's/:.*//'" },
  { "lpr", "-P", "-l", "/usr/libexec/lpc",
    "/usr/libexec/lpc status | grep '^...*:' | sed 's/:.*//'" },
  { "lpr", "-P", "-l", "/usr/sbin/lpc",
    "/usr/sbin/lpc status | grep '^...*:' | sed 's/:.*//'" },
};


PrinterQueues::PrinterQueues() : first(NULL), printercount(0)
{
	BuildList();
}

PrinterQueues::~PrinterQueues()
{
	while(first)
		delete first;
}


void PrinterQueues::IdentifyPrintSystem()
{
#ifdef HAVE_LIBCUPS
	printsystem=0;
#else
	int count=sizeof(PrintSystems)/sizeof(PrintSystem);

	for(int i=0;i<count;++i)
	{
		if (!access(PrintSystems[i].key_file, R_OK))
		{
			printsystem=i;
			i=count;
		}	
	}
#endif
}


void PrinterQueues::BuildList()
{
	FILE *pfile;
	char buf[256];

	IdentifyPrintSystem();

	if ((pfile = popen(PrintSystems[printsystem].scan_command, "r")))
    {
		while (fgets(buf, sizeof(buf), pfile) != NULL)
		{
			int i;
			for(i=strlen(buf)-1;i>0;--i)
			{
				if(buf[i]=='\n')
					buf[i]=0;
				if(buf[i]=='\r')
					buf[i]=0;
			}
			new PrinterInstance(*this,buf);
		}
		pclose(pfile);
	}
}


char *PrinterQueues::GetPrinterName(int idx)
{
	char *result=NULL;
	PrinterInstance *pi=first;
	while(pi && idx)
	{
		--idx;
		pi=pi->next;
	}
	if(pi)
	{
		result=strdup(pi->name);
	}
	return(result);
}


int PrinterQueues::GetPrinterCount()
{
	return(printercount);
}


char *PrinterQueues::GetPrintCommand(const char *printername)
{
	bool rawmode=true;
	int len=strlen(PrintSystems[printsystem].print_command);
	len+=strlen(PrintSystems[printsystem].queue_select);
	len+=strlen(PrintSystems[printsystem].raw_flag);
	len+=strlen(printername);

	char *driver=GetPrinterDriver(printername);
	if(driver)
	{
		if(strcmp("ps",driver)==0 || strcmp("ps2",driver)==0)
			rawmode=false;
	}

	char *cmd=(char *)malloc(len+15);
	sprintf(cmd,"%s %s %s %s",
		PrintSystems[printsystem].print_command,
		printername ? PrintSystems[printsystem].queue_select : "",
		printername ? printername : "",
		rawmode ? PrintSystems[printsystem].raw_flag : "");
	free(driver);
	return(cmd);
}


char *PrinterQueues::GetPrinterDriver(const char *printername)
{
#ifdef HAVE_LIBCUPS
	char *result=NULL;
	const char *ppdname=cupsGetPPD(printername);
	if(ppdname)
	{
		const char *cmd="cat %s | grep -i 'StpDriverName' | awk '{print $2}' | sed 's/\"//g'";
		int len=strlen(cmd)+strlen(ppdname)+2;
		char *cmd2=(char *)malloc(len);
		FILE *pfile;
		
		sprintf(cmd2,cmd,ppdname);
		
		if((pfile = popen(cmd2, "r")))
		{
			char buf[256];
			int i;
			if(fgets(buf, sizeof(buf), pfile))
			{
				for(i=strlen(buf)-1;i>0;--i)
				{
					if(buf[i]=='\n')
						buf[i]=0;
					if(buf[i]=='\r')
						buf[i]=0;
				}
				if(strlen(buf)>1)
					result=strdup(buf);
			}
			pclose(pfile);
		}
		free(cmd2);
	}
	if(!result)
		result=strdup("ps2");

	return(result);
#else
	return(NULL);
#endif
}
