#ifndef COLOURMANAGEMENT_H
#define COLOURMANAGEMENT_H

#include "imagesource.h"
#include "lcmswrapper.h"
#include "configdb.h"
#include "searchpath.h"

enum CMColourDevice
{
	CM_COLOURDEVICE_DISPLAY,
	CM_COLOURDEVICE_EXPORT,
	CM_COLOURDEVICE_PRINTER
};


class CMTransformFactory;

class ProfileManager : public ConfigDB, public SearchPath
{
	public:
	ProfileManager(ConfigFile *Configfile,const char *section);
	virtual ~ProfileManager();
	CMSProfile *GetProfile(const char *name);
	CMSProfile *GetProfile(CMColourDevice target);
	CMSProfile *GetDefaultProfile(IS_TYPE colourspace);
	void SetProfile(CMColourDevice *target,const char *name);
	void SetDefaultProfile(IS_TYPE colourspace,const char *name);
	CMTransformFactory *GetTransformFactory();
	protected:
	static ConfigTemplate Template[];
};


class CMTransformFactoryNode
{
	public:
	CMTransformFactoryNode(CMTransformFactory *header,CMSTransform *transform,MD5Digest &d1,MD5Digest &d2,int intent);
	~CMTransformFactoryNode();
	protected:
	CMTransformFactory *header;
	CMTransformFactoryNode *prev,*next;
	CMSTransform *transform;
	MD5Digest digest1;
	MD5Digest digest2;
	int intent;
	friend class CMTransformFactory;
};


class CMTransformFactory
{
	public:
	CMTransformFactory(ProfileManager &cm);
	~CMTransformFactory();
	CMSTransform *GetTransform(enum CMColourDevice target,CMSProfile *profile);
	CMSTransform *GetTransform(enum CMColourDevice target,IS_TYPE type);
	CMSTransform *Search(MD5Digest *srcdigest,MD5Digest *dstdigest,int intent);
	void Flush();
	protected:
	ProfileManager &manager;
	CMTransformFactoryNode *first;
	friend class CMTransformFactoryNode;
};

#endif
