/*
 * consumer.h - classes for writing to disk or piping raw binary data.
 *
 * Copyright (c) 2004 by Alastair M. Robinson
 * Distributed under the terms of the GNU General Public License -
 * see the file named "COPYING" for more details.
 *
 */

#ifndef CONSUMER_H
#define CONSUMER_H

#include <stdio.h>
#include <signal.h>

class Consumer
{
	public:
	virtual ~Consumer()
	{
	}
	virtual bool Write(const char *buffer, int length)=0;
};


class Consumer_File : public Consumer
{
	public:
	Consumer_File(char *filename)
	{
		if(!(file=fopen(filename,"wb")))
			throw "Can't open file for writing!";
	}
	~Consumer_File()
	{
		fclose(file);
	}
	bool Write(const char *buffer,int length)
	{
		int l=fwrite(buffer,length,1,file);
		return(l==length);
	}
	private:
	FILE *file;
};


class Consumer_Pipe : public Consumer
{
	public:
	Consumer_Pipe(char *command)
	{
		aborted=false;
		signal(SIGPIPE,&sighandler);
		if(!(pipe=popen(command,"w")))
			throw "Can't open pipe for writing!";
	}
	~Consumer_Pipe()
	{
		pclose(pipe);
	}
	bool Write(const char *buffer,int length)
	{
		fwrite(buffer,length,1,pipe);
		return(!aborted);
	}
	private:
	static void sighandler(int sig)
	{
		switch(sig)
		{
			case SIGPIPE:
				aborted=true;
				break;
			default:
				break;
		}
	}
	FILE *pipe;
	static bool aborted;
};

#endif
